import pytest
from orchestrator.types import SubscriptionLifecycle

from gso.products import ProductName
from gso.products.product_types.opengear import ImportedOpengear
from gso.products.product_types.site import Site
from test.workflows import (
    assert_complete,
    extract_state,
    run_workflow,
)


@pytest.fixture()
def workflow_input_data(faker, site_subscription_factory):
    return {
        "partner": "GEANT",
        "opengear_site": Site.from_subscription(site_subscription_factory()).site.site_name,
        "opengear_hostname": faker.hostname(),
        "opengear_wan_address": faker.ipv4(),
        "opengear_wan_netmask": faker.ipv4(),
        "opengear_wan_gateway": faker.ipv4(),
    }


@pytest.mark.workflow()
def test_create_imported_opengear_success(workflow_input_data):
    result, _, _ = run_workflow("create_imported_opengear", [workflow_input_data])
    state = extract_state(result)
    imported_opengear = ImportedOpengear.from_subscription(state["subscription_id"])

    assert_complete(result)
    assert imported_opengear.product.name == ProductName.IMPORTED_OPENGEAR
    assert imported_opengear.status == SubscriptionLifecycle.ACTIVE
